@@ -26,13 +26,16 @@
 #include "../utility_widgets/placeholder.h"
 #include "../utility_widgets/flowtablewithfields.h"
 
+namespace Glom
+{
+
 class Box_Data_Details : public Box_Data
 {
 public: 
   Box_Data_Details(bool bWithNavButtons = true);
   virtual ~Box_Data_Details();
 
-  virtual bool init_db_details(const Glib::ustring& table_name, const Gnome::Gda::Value& primary_key_value);
+  virtual bool init_db_details(const FoundSet& found_set, const Gnome::Gda::Value& primary_key_value);
   virtual bool refresh_data_from_database_with_primary_key(const Gnome::Gda::Value& primary_key_value);
   virtual bool refresh_data_from_database_blank();
 
@@ -41,6 +44,8 @@ public:
   virtual Gnome::Gda::Value get_primary_key_value() const; //Actual primary key value of this record.
   virtual Gnome::Gda::Value get_primary_key_value_selected(); //Value in the primary key's cell.
 
+  virtual void set_primary_key_value(const Gtk::TreeModel::iterator& /* row */, const Gnome::Gda::Value& value);
+
   virtual Gnome::Gda::Value get_entered_field_data(const sharedptr<const LayoutItem_Field>& field) const;
   virtual void set_entered_field_data(const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& value);
   virtual void set_entered_field_data(const Gtk::TreeModel::iterator& row, const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& value);
@@ -132,4 +137,6 @@ protected:
   type_signal_requested_related_details m_signal_requested_related_details;
 };
 
+} //namespace Glom
+
 #endif //BOX_DATA_DETAILS_H
