@@ -24,6 +24,7 @@
 #include <bakery/App/App_Gtk.h> //For util_bold_message().
 
 //#include <libgnome/gnome-i18n.h>
+#include <gtkmm/togglebutton.h>
 #include <glibmm/i18n.h>
 
 namespace Glom
@@ -37,11 +38,19 @@ Dialog_Layout_List_Related::Dialog_Layout_List_Related(BaseObjectType* cobject,
   m_button_field_add(0),
   m_button_field_delete(0),
   m_button_field_edit(0),
-  m_combo_relationship(0)
+  m_combo_relationship(0),
+  m_checkbutton_show_child_relationships(0),
+  m_radio_navigation_automatic(0),
+  m_radio_navigation_specify(0),
+  m_label_navigation_automatic(0),
+  m_combo_navigation_specify(0)
 {
   refGlade->get_widget_derived("combo_relationship_name", m_combo_relationship);
   m_combo_relationship->signal_changed().connect(sigc::mem_fun(*this, &Dialog_Layout_List_Related::on_combo_relationship_changed));
 
+  refGlade->get_widget("checkbutton_show_child_relationships", m_checkbutton_show_child_relationships);
+  m_checkbutton_show_child_relationships->signal_toggled().connect(sigc::mem_fun(*this, &Dialog_Layout_List_Related::on_checkbutton_show_child_relationships));
+
   refGlade->get_widget("treeview_fields", m_treeview_fields);
   if(m_treeview_fields)
   {
@@ -91,8 +100,24 @@ Dialog_Layout_List_Related::Dialog_Layout_List_Related(BaseObjectType* cobject,
   refGlade->get_widget("button_field_formatting", m_button_field_formatting);
   m_button_field_formatting->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_Layout_List_Related::on_button_field_formatting) );
 
+  refGlade->get_widget("radiobutton_navigation_automatic", m_radio_navigation_automatic);
+  refGlade->get_widget("label_navigation_automatic", m_label_navigation_automatic);
+  make_sensitivity_depend_on_toggle_button(*m_radio_navigation_automatic, *m_label_navigation_automatic);
+
+  refGlade->get_widget("radiobutton_navigation_specify", m_radio_navigation_specify);
+  refGlade->get_widget_derived("combobox_navigation_specify", m_combo_navigation_specify);
+  make_sensitivity_depend_on_toggle_button(*m_radio_navigation_specify, *m_combo_navigation_specify);
+  m_combo_navigation_specify->signal_changed().connect(sigc::mem_fun(*this, &Dialog_Layout_List_Related::on_combo_navigation_specific_changed));
+
+ 
+  m_modified = false;
 
   show_all_children();
+
+  //This entry must be in the Glade file, because it's used by the base class,
+  //but we don't want it here, because it is confusing when dealing with relationships:
+  if(m_entry_table_title)
+    m_entry_table_title->hide(); // We don't use this (it's from the base class).
 }
 
 Dialog_Layout_List_Related::~Dialog_Layout_List_Related()
@@ -117,36 +142,44 @@ void Dialog_Layout_List_Related::update_ui(bool including_relationship_list)
 
   m_modified = false;
 
+  const Glib::ustring related_table_name = m_portal->get_table_used(Glib::ustring() /* parent table - not relevant*/);
+
   //Update the tree models from the document
   Document_Glom* document = get_document();
   if(document)
   {
-    const Glib::ustring parent_relationship_name = m_portal->get_relationship()->get_name();
-    const Glib::ustring parent_relationship_title = m_portal->get_relationship()->get_title_or_name();
-
     //Fill the relationships combo:
     if(including_relationship_list)
     {
-      const Document_Glom::type_vecRelationships vecRelationships = document->get_relationships(m_portal->get_relationship()->get_from_table());
-      m_combo_relationship->set_relationships(vecRelationships); //We don't show the optional parent table because portal use _only_ relationships, of course.
+      bool show_child_relationships = m_checkbutton_show_child_relationships->get_active();
+        
+      //For the showing of child relationships if necessary:
+      if(!show_child_relationships && m_portal->get_related_relationship())
+      {
+        show_child_relationships = true;
+      }
+
+      m_combo_relationship->set_relationships(document, m_portal->get_relationship()->get_from_table(), show_child_relationships, false /* don't show parent table */); //We don't show the optional parent table because portal use _only_ relationships, of course.
+
+      if(show_child_relationships != m_checkbutton_show_child_relationships->get_active())
+      {
+         m_checkbutton_show_child_relationships->set_active(show_child_relationships);
+      }
     }
 
     //Set the table name and title:
     sharedptr<LayoutItem_Portal> portal_temp = m_portal;
-    m_combo_relationship->set_selected_relationship(m_portal->get_relationship()); 
-
-    m_entry_table_title->set_text( parent_relationship_title );
+    m_combo_relationship->set_selected_relationship(m_portal->get_relationship(), m_portal->get_related_relationship()); 
 
     Document_Glom::type_mapLayoutGroupSequence mapGroups;
     mapGroups[0] = m_portal;
-    document->fill_layout_field_details(m_portal->get_relationship()->get_to_table(), mapGroups); //Update with full field information.
+    document->fill_layout_field_details(related_table_name, mapGroups); //Update with full field information.
 
     //If no information is stored in the document, then start with something:
 
     if(mapGroups.empty())
     {
-      const Glib::ustring table_name = m_portal->get_relationship()->get_to_table();
-      Document_Glom::type_vecFields table_fields = document->get_table_fields(table_name);
+      Document_Glom::type_vecFields table_fields = document->get_table_fields(related_table_name);
 
       sharedptr<LayoutGroup> group = sharedptr<LayoutGroup>::create();
       group->set_name("main");
@@ -196,6 +229,61 @@ void Dialog_Layout_List_Related::update_ui(bool including_relationship_list)
     treeview_fill_sequences(m_model_fields, m_ColumnsFields.m_col_sequence); //The document should have checked this already, but it does not hurt to check again.
   }
 
+  //Show the navigation information:
+  //const Document_Glom::type_vecRelationships vecRelationships = document->get_relationships(m_portal->get_relationship()->get_from_table());
+  m_combo_navigation_specify->set_relationships(document, related_table_name, true /* show related relationships */, false /* don't show parent table */); //TODO: Don't show the hidden tables, and don't show relationships that are not used by any fields.
+  //m_combo_navigation_specify->set_display_parent_table(""); //This would be superfluous, and a bit confusing.
+
+  bool navigation_is_automatic = false;
+  bool navigation_specific_main = false;
+  sharedptr<UsesRelationship> navrel = m_portal->get_navigation_relationship_specific(navigation_specific_main);
+  if(navigation_specific_main)
+    m_combo_navigation_specify->set_selected_parent_table(related_table_name);
+  else if(navrel)
+  {
+    //std::cout << "debug navrel=" << navrel->get_relationship()->get_name() << std::endl;
+    m_combo_navigation_specify->set_selected_relationship(navrel->get_relationship(), navrel->get_related_relationship());
+  }
+  else
+  {
+    navigation_is_automatic = true;
+
+    sharedptr<const Relationship> none;
+    m_combo_navigation_specify->set_selected_relationship(none);
+  }
+
+  //Set the appropriate radio button:
+  //std::cout << "debug: navigation_is_automatic=" << navigation_is_automatic << std::endl;
+  m_radio_navigation_automatic->set_active(navigation_is_automatic); 
+  m_radio_navigation_specify->set_active(!navigation_is_automatic); 
+
+
+  //Describe the automatic navigation:
+  sharedptr<const UsesRelationship> relationship_navigation_automatic;
+  bool navigation_automatic_main = false;
+  relationship_navigation_automatic = get_portal_navigation_relationship_automatic(m_portal, navigation_automatic_main);
+  Glib::ustring automatic_navigation_description;
+  
+  if(navigation_automatic_main)
+    automatic_navigation_description = m_portal->get_relationship_name_used();
+  else if(relationship_navigation_automatic)
+  {
+    automatic_navigation_description = m_portal->get_relationship_name_used();
+
+    if(relationship_navigation_automatic->get_has_relationship_name())
+      automatic_navigation_description += ("::" + relationship_navigation_automatic->get_relationship_name());
+
+    if(relationship_navigation_automatic->get_has_related_relationship_name())
+      automatic_navigation_description += ("::" + relationship_navigation_automatic->get_related_relationship_name());
+  }
+
+  if(automatic_navigation_description.empty())
+  {
+    automatic_navigation_description = _("None: No visible tables are specified by the fields.");
+  }
+
+  m_label_navigation_automatic->set_text(automatic_navigation_description);
+
   m_modified = false;
 }
 
@@ -283,31 +371,52 @@ void Dialog_Layout_List_Related::save_to_document()
       }
     }
 
-    Document_Glom* document = get_document();
-    if(document)
+    if(m_radio_navigation_specify->get_active())
     {
-      //Save the relationship title, which will be used elsewhere too:
-      m_portal->get_relationship()->set_title(m_entry_table_title->get_text());
-      document->set_relationship(m_table_name, m_portal->get_relationship());
+      sharedptr<Relationship> rel, rel_related;
+      rel = m_combo_navigation_specify->get_selected_relationship(rel_related);
+
+      bool specify_main = (!rel && !rel_related);
+      sharedptr<UsesRelationship> uses_rel = sharedptr<UsesRelationship>::create();
+      uses_rel->set_relationship(rel);
+      uses_rel->set_related_relationship(rel_related);
+
+      m_portal->set_navigation_relationship_specific(specify_main, uses_rel);
+      //std::cout << "debug99 main=specify_main" << ", relationship=" << (rel ? rel->get_name() : "none") << std::endl;
+    }
+    else
+    {
+      //std::cout << "debug: set_navigation_relationship_specific(false, none)" << std::endl;
+      sharedptr<UsesRelationship> none;
+      m_portal->set_navigation_relationship_specific(false, none);
     }
   }
 }
 
+void Dialog_Layout_List_Related::on_checkbutton_show_child_relationships()
+{
+  update_ui();
+}
+
 void Dialog_Layout_List_Related::on_combo_relationship_changed()
 {
   if(!m_portal)
     return;
 
-  sharedptr<Relationship> relationship = m_combo_relationship->get_selected_relationship();
+  sharedptr<Relationship> relationship_related;
+  sharedptr<Relationship> relationship = m_combo_relationship->get_selected_relationship(relationship_related);
   if(relationship)
   {
     //Clear the list of fields if the relationship has changed, because the fields could not possible be correct for the new table:
     bool relationship_changed = false;
     const Glib::ustring old_relationship_name = glom_get_sharedptr_name(m_portal->get_relationship());
-    if(old_relationship_name !=  glom_get_sharedptr_name(relationship))
+    const Glib::ustring old_relationship_related_name = glom_get_sharedptr_name(m_portal->get_related_relationship());
+    if( (old_relationship_name != glom_get_sharedptr_name(relationship)) ||
+        (old_relationship_related_name != glom_get_sharedptr_name(relationship_related)) )
       relationship_changed = true;
 
     m_portal->set_relationship(relationship);
+    m_portal->set_related_relationship(relationship_related);
 
     if(relationship_changed)
       m_portal->remove_all_items();
@@ -327,7 +436,10 @@ void Dialog_Layout_List_Related::on_treeview_fields_selection_changed()
 void Dialog_Layout_List_Related::on_button_add_field()
 {
   //Get the chosen field:
-  sharedptr<LayoutItem_Field> field = offer_field_list(m_portal->get_relationship()->get_to_table(), this);
+  //std::cout << "debug: related relationship=" << glom_get_sharedptr_name(m_portal->get_related_relationship()) << std::endl;
+  //std::cout << "debug table used =" << m_portal->get_table_used(m_table_name) << std::endl;
+
+  sharedptr<LayoutItem_Field> field = offer_field_list(m_portal->get_table_used(m_table_name), this);
   if(field)
   {
     //Add the field details to the layout treeview:
@@ -367,6 +479,7 @@ void Dialog_Layout_List_Related::on_button_delete()
 
 sharedptr<Relationship> Dialog_Layout_List_Related::get_relationship() const
 {
+  std::cout << "debug: I wonder if this function is used." << std::endl;
   return m_combo_relationship->get_selected_relationship();
 }
 
@@ -402,7 +515,7 @@ void Dialog_Layout_List_Related::on_button_edit_field()
       sharedptr<LayoutItem_Field> field = row[m_ColumnsFields.m_col_layout_item];
 
       //Get the chosen field:
-      sharedptr<LayoutItem_Field> field_chosen = offer_field_list(field, m_portal->get_relationship()->get_to_table(), this);
+      sharedptr<LayoutItem_Field> field_chosen = offer_field_list(field, m_portal->get_table_used(m_table_name), this);
       if(field_chosen)
       {
         //Set the field details in the layout treeview:
@@ -447,7 +560,7 @@ void Dialog_Layout_List_Related::on_button_field_formatting()
           Gtk::TreeModel::Row row = *iter;
           sharedptr<LayoutItem_Field> field = row[m_ColumnsFields.m_col_layout_item];
 
-          dialog->set_field(field, m_portal->get_relationship()->get_to_table());
+          dialog->set_field(field, m_portal->get_table_used(Glib::ustring() /* parent table - not relevant */));
           dialog->set_transient_for(*this);
           int response = dialog->run();
           dialog->hide();
@@ -476,5 +589,11 @@ sharedptr<LayoutItem_Portal> Dialog_Layout_List_Related::get_portal_layout()
   return m_portal;
 }
 
+void Dialog_Layout_List_Related::on_combo_navigation_specific_changed()
+{
+  m_modified = true;
+}
+
+
 } //namespace Glom
 
